<!--ts-->
   * [Web Application for Auto Reporting](#web-application-for-auto-reporting)
      * [TODO](#todo)
      * [How to Create a HTML Version of this File.](#how-to-create-a-html-version-of-this-file)
      * [Access to reportsvr01 and git Setup](#access-to-reportsvr01-and-git-setup)
      * [Create Python 3 Environment, Upgrade pip, Install pip Packages Required for KPY](#create-python-3-environment-upgrade-pip-install-pip-packages-required-for-kpy)
      * [Install KPY on VM reportsvr01](#install-kpy-on-vm-reportsvr01)
      * [Running KPY](#running-kpy)
      * [Problems when Running KPY](#problems-when-running-kpy)
         * [Problem: KeyError: 'LDAP_USER'](#problem-keyerror-ldap_user)
         * [Solution: The bash environment variable LDAP_USER is not set.](#solution-the-bash-environment-variable-ldap_user-is-not-set)
         * [Problem: ValueError: Expecting ',' delimiter](#problem-valueerror-expecting--delimiter)
         * [Solution:](#solution)
      * [Bottle Web Application - Install Requirements and Run](#bottle-web-application---install-requirements-and-run)
         * [Install the bottle micro web framework:](#install-the-bottle-micro-web-framework)
         * [Install wsgi Stuff](#install-wsgi-stuff)
         * [Testing on your local machine](#testing-on-your-local-machine)
         * [Testing on Virtual Machine reportsvr01](#testing-on-virtual-machine-reportsvr01)
         * [Production on Virtual Machine reportsvr01](#production-on-virtual-machine-reportsvr01)
      * [Problems running Bottle Web Application](#problems-running-bottle-web-application)
      * [Configure firewalld](#configure-firewalld)
      * [Configure nginx](#configure-nginx)
      * [Self Signed Cert](#self-signed-cert)
      * [Stuff to Sort](#stuff-to-sort)
      * [References](#references)

<!-- Added by: mlake, at: 2018-05-21T09:56+10:00 -->

<!--te-->

# Web Application for Auto Reporting

This web application provides a web site which allows members of eReserach to run the 
program that gathers the emil interactions across the faculties. This would be a better 
alternative to having someone walk around the office with a laptop getting users to 
run the script. 
The web app is being developed on the Virtual Machine ```reportsvr01```. 
It is (or will be) available at <https://138.25.37.49/kpy/>


## TODO

  - Install this to /home/autoreporting/ 
  - Create /var/www/kpy/static structure on reportsvr01.
  - Get cert and setup https.
  - Implement check_login().
  - Set cookie or persistent state to remember your email address.
  - Automatically set current month & year.
  - Reports page to show your previous interactions i.e. images in static/output/
    and maybe cron job to prune those every 6 months.

There is a lot off stuff that was copied and pasted here during Mike Lakes development 
of this that should be "genericised" to be less specific e.g. removal of staff number etc.


## How to Create a HTML Version of this File.

    pandoc --smart --toc -H README.css -s -o README_webapp.html README_webapp.md


## Access to reportsvr01 and git Setup

reportsvr01 is a Centos 7.4 VM.
To login connect to 138.25.37.49 via ssh and use staff number/email password. Also 
use -X as graphics needs to be displayed.
In the example below 950654 is Mike Lakes staff number.

    $ ssh -X 950654@138.25.37.49
    950654@138.25.37.49's password: 
    Creating home directory for 950654.
    /usr/bin/id: cannot find name for group ID 950654

    [950654@reportsvr01 ~]$ pwd
    /home/LDAP/950654

Copied over my private key:
    
    $ scp mikes_cluster.pri 950654@138.25.37.49:/home/LDAP/950654/.ssh/

Before you checkout the KPY code create a `.gitconfig` in my your home directory:

    [950654@reportsvr01 ~]$ vi .gitconfig

Thenceforth to use git start the `ssh-agent` and load your private key:

    [950654@reportsvr01]$ eval `ssh-agent` 
    
    [950654@reportsvr01]$ ssh-add ~/.ssh/mikes_cluster.pri 
    Identity added: /home/LDAP/950654/.ssh/mikes_cluster.pri 
    $ 


## Create Python 3 Environment, Upgrade pip, Install pip Packages Required for KPY

Here we will create a python 3 virtual environment. 
Then we install markerlib, ldap3, networkx, matplotlib, pandas and pyzmail which are required for the 
KPY application.

First we need to install python34-tkinter as a yum package.
OK 

    sudo yum install python34-tkinter 

Check we have a python 3 installed. 

    [950654@reportsvr01 ~]$ yum list installed | grep python3
    python34.x86_64                   3.4.5-3.el7                         @epel  

Create python virtualenv.

    [950654@reportsvr01 virtualenvs]$ virtualenv -p python3 autoreporting
    Running virtualenv with interpreter /usr/bin/python3
    Using base prefix '/usr'
    New python executable in /home/LDAP/950654/virtualenvs/autoreporting/bin/python3
    Also creating executable in /home/LDAP/950654/virtualenvs/autoreporting/bin/python
    Installing setuptools, pip, wheel...done.
    [950654@reportsvr01 virtualenvs]$ 

Invoke this virtualenv:

    [950654@reportsvr01 virtualenvs]$ source autoreporting/bin/activate
    (autoreporting) [950654@reportsvr01 virtualenvs]$ 

Upgrade pip:

    (autoreporting)KPY/$ pip install --upgrade pip
    You are using pip version 6.0.8, however version 9.0.1 is available.
    You should consider upgrading via the 'pip install --upgrade pip' command.
    Collecting pip from https://pypi.python.org/packages/b6/ac/....... 
      Using cached pip-9.0.1-py2.py3-none-any.whl
    Installing collected packages: pip
      Found existing installation: pip 6.0.8
        Uninstalling pip-6.0.8:
          Successfully uninstalled pip-6.0.8
    
    Successfully installed pip-9.0.1
    (autoreporting)KPY/$ 

We need to install some python packages into our new virtual python environemnt.

    (autoreporting) [950654@reportsvr01 KPY]$ pip search markerlib
    (autoreporting) [950654@reportsvr01 KPY]$ pip install  markerlib
    (autoreporting) [950654@reportsvr01 KPY]$ pip install ldap3
    (autoreporting) [950654@reportsvr01 KPY]$ pip install networkx
    (autoreporting) [950654@reportsvr01 KPY]$ pip install matplotlib
    (autoreporting) [950654@reportsvr01 KPY]$ pip install pandas
    (autoreporting) [950654@reportsvr01 KPY]$ pip install pyzmail

Notes: markerlib (0.6.0) is a compiler for PEP 345 environment markers.  
pyzmail required markerlib so needed to be installed after that. 

How to list outdated pip modules: `pip list --outdated --format=freeze`
Also `pip freeze > requirements.txt` and `pip install -r requirements.txt`


## Install KPY on VM reportsvr01

Checkout code:
    
    [950654@reportsvr01 git]$ git clone git@codeine.research.uts.edu.au:eresearch/KPY.git
    Cloning into 'KPY'...
    $ 


## Running KPY

You need to set a couple of bash environment variables 

    $ cd git/KPY    

    [950654@reportsvr01]$ source ~/virtualenvs/autoreporting/bin/activate
    (autoreporting) [950654@reportsvr01]$ 

    (autoreporting)KPY/$ python engagement_individual.py outlook.office365.com  Your.Email@uts.edu.au
        -f "Sent" -q "SINCE 01-Apr-2017 BEFORE 01-May-2017" -m groups.json

Note: Needs -X when sshing. Now runs OK


## Problems when Running KPY

### Problem: KeyError: 'LDAP_USER'

    (autoreporting)KPY/$ python engagement_individual.py outlook.office365.com  Mike.Lake@uts.edu.au -f "Sent" -q "SINCE 01-Dec-2016 BEFORE 01-Jan-2017" -m groups.json
    Password:
    Traceback (most recent call last):
      File "engagement_individual.py", line 45, in <module>
        main()
      File "engagement_individual.py", line 37, in main
        iaction = interactions.Interactions(args)
      File "/home/mlake/git/KPY/interactions/interactions.py", line 27, in __init__
        self.ldap_connection = ldap3.Connection(server, os.environ['LDAP_USER'] , \
      File "/home/mlake/virtualenvs/autoreporting/lib64/python3.4/os.py", line 633, in __getitem__
        raise KeyError(key) from None
    KeyError: 'LDAP_USER'
    $ 

### Solution: The bash environment variable LDAP_USER is not set.

    export LDAP_USER='cn=eResearch Proxy Agent,ou=System Users,o=UTS'

To set the required bash variables use this bash script to run the query:
 
    #!/bin/bash
    # # DO NOT CHECK THIS INTO GIT !
    # Run KPY for Mike Lake
    source ~/virtualenvs/autoreporting/bin/activate
    export LDAP_USER='cn=eResearch Proxy Agent,ou=System Users,o=UTS'
    export LDAP_PASSWD='top secret'
    python engagement_individual.py outlook.office365.com Mike.Lake@uts.edu.au \
        -f "Sent" \
        -q "SINCE 01-Apr-2017 BEFORE 09-May-2017" \
        -m groups.json

### Problem: ValueError: Expecting ',' delimiter

    python engagement_individual.py outlook.office365.com Mike.Lake@uts.edu.au \
        -f "Sent" \
        -q "SINCE 01-Apr-2018 BEFORE 01-May-2018" \
        -m groups.json
    
    Traceback (most recent call last):
      File "engagement_individual.py", line 46, in <module>
        main()
      File "engagement_individual.py", line 39, in main
        iaction = interactions.Interactions(args)
      File "/home/LDAP/950654/git/KPY/interactions/interactions.py", line 30, in __init__
        self.dept_lookup = json.load(open(args.mapping))  if args.mapping else {}
      File "/usr/lib64/python3.4/json/__init__.py", line 268, in load
        parse_constant=parse_constant, object_pairs_hook=object_pairs_hook, **kw)
      File "/usr/lib64/python3.4/json/__init__.py", line 318, in loads
        return _default_decoder.decode(s)
      File "/usr/lib64/python3.4/json/decoder.py", line 343, in decode
        obj, end = self.raw_decode(s, idx=_w(s, 0).end())
      File "/usr/lib64/python3.4/json/decoder.py", line 359, in raw_decode
        obj, end = self.scan_once(s, idx)
    ValueError: Expecting ',' delimiter: line 7 column 2 (char 318)

### Solution: 


## Bottle Web Application - Install Requirements and Run

### Install the bottle micro web framework:
    
We need the Python development headers for uwsgi so install this now. 
It needs to be installed *before* uwsgi.
     
    reportsvr01 ~$ sudo yum install python3-devel
    OK

We will need a proper web server so lets install nginx now.

    [950654@reportsvr01 ~]$ sudo yum install nginx

    Installing:
      nginx                          1:1.10.2-1.el7   epel   505 k
    Installing for dependencies:
      nginx-all-modules              1:1.10.2-1.el7   epel    16 k
      nginx-filesystem               1:1.10.2-1.el7   epel    17 k
      nginx-mod-http-geoip           1:1.10.2-1.el7   epel    23 k
      nginx-mod-http-image-filter    1:1.10.2-1.el7   epel    26 k
      nginx-mod-http-perl            1:1.10.2-1.el7   epel    35 k
      nginx-mod-http-xslt-filter     1:1.10.2-1.el7   epel    25 k
      nginx-mod-mail                 1:1.10.2-1.el7   epel    53 k
      nginx-mod-stream               1:1.10.2-1.el7   epel    46 k

Now install the pip modules that we need for the web application.

    (autoreporting)KPY/$ pip install bottle
    OK 

### Install wsgi Stuff

    (autoreporting)KPY/$ pip install uwsgi
    OK

TODO - up to here ----------------------------------- 

Installed: 

uwsgi                   Provides /usr/sbin/uwsgi and other files in /etc/ and /usr/share/doc/uwsgi
uwsgi-plugin-python     Only provides /usr/lib64/uwsgi/python_plugin.so
uwsgi-plugin-common     Pulled in as a dependeny of uwsgi-plugin-python 

sudo systemctl start uwsgi.service
sudo systemctl enable uwsgi.service
    Created symlink from /etc/systemd/system/multi-user.target.wants/uwsgi.service to /usr/lib/systemd/system/uwsgi.service.

Create /etc/uwsgi.ini  /etc/uwsgi.d/kpy.ini  /etc/uwsgi.d/hello.ini 


end -------------------------------------------------


### Testing on your local machine 

This test will not use the nginx web server. It will use bottle's inbuilt web server. 
Make sure `TEST = True`

    KPY/$ source ~/virtualenvs/autoreporting/bin/activate
    (autoreporting)KPY/$ 
    
    export LDAP_USER='cn=eResearch Proxy Agent,ou=System Users,o=UTS'
    export LDAP_PASSWD='top secret'

    (autoreporting)KPY/$ ./autoreport_web.py
    Bottle v0.12.13 server starting up (using WSGIRefServer())...
    Listening on http://localhost:8080/
    Hit Ctrl-C to quit.

Go to location http://localhost:8080/ in your web browser.
Enter your email and LDAP password and click [Plot my Interactions].
After a while a graph will appear of your email interactions for each faculty.

### Testing on Virtual Machine reportsvr01

Using bottle's inbuilt webserver: Make sure `TEST = True`. 

    (autoreporting) [950654@reportsvr01 KPY]$ ./autoreport_web.py 
    Bottle v0.12.13 server starting up (using WSGIRefServer())...
    Listening on http://localhost:8080/
    Hit Ctrl-C to quit.

Using WSGI: Make sure `TEST = False`. 

Because we are in the virtual python environment a `which uwsgi` 
will report `~/virtualenvs/autoreporting/bin/uwsgi` so we don't need to use its full path.
We need to use port `8080` as we are not using sudo in testing.
    
    $ sudo systemctl stop firewalld

    (autoreporting) [950654@reportsvr01 KPY]$ 
        uwsgi --http :8080 --wsgi-file autoreport_web.py

Go to <http://138.25.37.49:8080/>

This works OK on Mikes laptop:

    $ uwsgi --http 127.0.0.1:8080 --wsgi-file autoreport_web.py

### Production on Virtual Machine reportsvr01

To use port 8080 we need to stop the firewall.

    $ sudo systemctl stop firewalld
    
    KPY]$ /home/LDAP/950654/virtualenvs/autoreporting/bin/uwsgi --http :8080 --wsgi-file autoreport_web.py


## Problems running Bottle Web Application

    (autoreporting) [950654@reportsvr01 KPY]$ 
    uwsgi --http :8080 --wsgi-file autoreport_web.py
    *** Starting uWSGI 2.0.15 (64bit) on [Fri May 11 11:50:11 2018] ***
    .....
    unable to load configuration from uwsgi
    $ 


## Configure firewalld

This machine is running package firewalld-0.4 (/usr/sbin/firewalld)
System configuration settings in /etc/firewalld/.

    /etc/firewalld/zones/public.xml

    dnsmasq
    dhclient

    [root@reportsvr01 firewalld]# firewall-cmd --permanent --zone=public --add-port=80/tcp
    success
    #

    [root@reportsvr01 firewalld]# firewall-cmd --reload
    success
    #

https://unix.stackexchange.com/questions/110558/su-with-error-x11-connection-rejected-because-of-wrong-authentication

[950654@reportsvr01 ~]$ echo $DISPLAY
138.25.37.49:11.0
[950654@reportsvr01 ~]$ xauth list

[950654@reportsvr01 ~]$ sudo su -
[root@reportsvr01 ~]# xauth merge /home/LDAP/950654/.Xauthority
[root@reportsvr01 ~]# firewall-config 


Ref: https://www.digitalocean.com/community/tutorials/how-to-set-up-a-firewall-using-firewalld-on-centos-7


## Configure nginx

We will need to setup a TLS enabled server.  We will need to create a self-signed cert for this.

An example for some of the things to set (but not TLS) 
is in ```/home/mlake/git/choice/``

    $ sudo mkdir -p /var/www/html/kpy/static/output


    reportsvr01 ~]$ sudo systemctl start nginx

    reportsvr01 ~]$ sudo systemctl status nginx

Go to <http://138.25.37.49/kpy/>

(reportsvr01 is centos 7. For Centos 6 only: `service nginx start` and `chkconfig nginx on`.) 


## Self Signed Cert

There is a script at /etc/pki/nginx/tls/certs/make-dummy-cert from the openssl package.


    $ sudo firewall-cmd --add-service=https
    $ sudo firewall-cmd --runtime-to-permanent



https://www.digitalocean.com/community/tutorials/how-to-create-a-self-signed-ssl-certificate-for-nginx-on-centos-7

https://stackoverflow.com/questions/10175812/how-to-create-a-self-signed-certificate-with-openssl
https://toggen.com.au/blog/it-tips/using-make-dummy-cert-to-create-a-self-signed-certificate-to-https-enable-an-nginx-served-website-in-centos-and-other-mouthfuls



## Stuff to Sort

This failed with errors as there is no try surrounding the getting of the data
which is the passed as an empty list to plot.

    (autoreporting)KPY/$ python engagement_individual.py outlook.office365.com Mike.Lake@uts.edu.au -f "Sent" -q "SINCE 01-Dec-2016 BEFORE 01-Jan-2017" -m groups.json

This works OK

    (autoreporting)KPY/$ python engagement_individual.py outlook.office365.com Mike.Lake@uts.edu.au -f "Sent" -q "SINCE 01-Apr-2017 BEFORE 01-May-2017" -m groups.json

Password:

    :TO: sablokg@gmail.comConsidering email to sablokg@gmail.com @ False
    :TO: brent.a.bevear@student.uts.edu.auConsidering email to brent.a.bevear@student.uts.edu.au @ Students & alumni
    :TO: Stacey.Trevathan-Tackett@uts.edu.auConsidering email to stacey.trevathan-tackett@uts.edu.au @ False
    :TO: LeonardoAndres.Hardtke@uts.edu.auConsidering email to leonardoandres.hardtke@uts.edu.au @ Science
    :TO: LeonardoAndres.Hardtke@uts.edu.auConsidering email to leonardoandres.hardtke@uts.edu.au @ Science
    ..........
    :TO: Steven.Djordjevic@uts.edu.auConsidering email to steven.djordjevic@uts.edu.au @ Science
    :TO: Piklu.Bhattacharya@uts.edu.auConsidering email to piklu.bhattacharya@uts.edu.au @ Science
    :TO: matt.demaere@gmail.comConsidering email to matt.demaere@gmail.com @ False
    
                                 Library etc  Students & alumni  Science  \
    Library etc                          0.0                0.0      1.0   
    Students & alumni                    0.0                0.0     27.0   
    Science                              1.0               27.0     83.0   
    University Casual Academics          0.0                0.0      3.0   
    DVCR Dept                            0.0                0.0      2.0   
    
                                 University Casual Academics  DVCR Dept  
    Library etc                                          0.0        0.0  
    Students & alumni                                    0.0        0.0  
    Science                                              3.0        2.0  
    University Casual Academics                          0.0        0.0  
    DVCR Dept                                            0.0        0.0  
                          DVCR Dept  Library etc  Science  Students & alumni  \
    Mike.Lake@uts.edu.au        2.0          1.0     83.0               27.0   
    
                          University Casual Academics  
    Mike.Lake@uts.edu.au                          3.0  
                          DVCR Dept  Library etc  Science  Students & alumni  \
    Mike.Lake@uts.edu.au        2.0          1.0     83.0               27.0   
    
                          University Casual Academics  
    Mike.Lake@uts.edu.au                          3.0  


Error appeared: "Couldnt get a file descriptor referring to the console"
This is because X forwarding is not set.

Current directory is /home/mlake/git/autoreporting/KPY/Output/SINCE 01-Apr-2017 BEFORE 01-May-2017/Mike.Lake_at_uts.edu.au

    drwxr-xr-x    3 mlake    mlake	 4096 May  8 14:45 ../
    -rw-r--r--    1 mlake    mlake	  120 May  8 14:45 interaction_data.csv
    -rw-r--r--    1 mlake    mlake	16802 May  8 14:45 interaction_data.png


## References

Bottle: http://bottlepy.org

Bottle Tutorial: http://bottle.readthedocs.org/en/latest/tutorial_app.html

Bottle template engine: http://bottlepy.org/docs/dev/stpl.html

Bottle static fles:

    http://stackoverflow.com/questions/10486224/bottle-static-files/10487336
    http://stackoverflow.com/questions/10486224/bottle-static-files/10487336
    http://stackoverflow.com/questions/10486224/bottle-static-files

UWSGI: https://uwsgi-docs.readthedocs.io/en/latest/WSGIquickstart.html

https://www.digitalocean.com/community/tutorials/how-to-set-up-uwsgi-and-nginx-to-serve-python-apps-on-centos-7
https://www.digitalocean.com/community/tutorials/how-to-serve-flask-applications-with-uwsgi-and-nginx-on-centos-7

https://www.digitalocean.com/community/tutorials/how-to-install-nginx-on-centos-7
https://www.godaddy.com/garage/tech/config/how-to-install-and-configure-nginx-on-centos-6/
https://www.digitalocean.com/community/tutorials/how-to-configure-the-nginx-web-server-on-a-virtual-private-server


