#!/usr/bin/env python3
# coding: utf-8

import pandas as pd
import matplotlib.pyplot as plt
import matplotlib
import requests
import yaml
#import base64
import os.path
import datetime
import dateutil
import re
import sys

import pystache
from kpy.project import Project, Projects
from kpy.jobs import Jobs
from kpy.gitlab import Gitlab


import argparse

DEBUG = False

CONFIG = './config.yml'


class HighlightsReport(object):
    """Report text to be rendered into a mustache template"""
    def __init__(self, cf):
        self.load_config(cf)
        current_date = datetime.datetime.now().date()
        self.period = {}
        if 'yearly' in cf:
            bits = cf_general['yearly'].split('-')
            month = int(bits[0])
            day = int(bits[1])
            self.period['start'] = datetime.date(current_date.year - 1, month, day)
            self.period['end'] = datetime.date(current_date.year, month, day)
        else:
            self.period['end'] = datetime.date(current_date.year, current_date.month, self.cf['day'])
            prev_month = current_date.month - 1
            prev_year = current_date.year
            if prev_month < 1:
                prev_month = 12
                prev_year -= 1
            self.period['start'] = datetime.date(prev_year, prev_month, self.cf['day'])
            print("********", self.period['start'], self.period['end'])
            
        matplotlib.style.use('bmh')


    def load_config(self, conffile):
        self.config = None

        with open(conffile) as cf:
            try:
                self.config = yaml.load(cf)
            except yaml.YAMLError as exc:
                print("%s parse error: %s" % ( CONFIG, exc ))
                if hasattr(exc, 'problem_mark'):
                    mark = exc.problem_mark
                    print("Error position: (%s:%s)" % (mark.line + 1, mark.column + 1))
        if not self.config:
            print("Config error")
            sys.exit(-1)
    
        self.cf = self.config['General']
    #self.cf_sc = config['ServiceConnect']
    #self.cf_projects = config['Projects']
                        
    def faculty(self, ou):
        print("OU", ou, end=" ")
        if ou in self.cf['faculties']:
            print("found")
            return self.cf['faculties'][ou]
        else:
            print("not")
            return 'Other'

    def project_summaries(self):
        """View method which returns a list of { project: Project }"""
        print(self.projects)
        if self.projects:
            return [ { 'project': p } for p in self.projects.projects ]
        else:
            return []

        
    def load(self):
        if self.config['ServiceConnect']['active']:
            self.jobs = Jobs(self, self.config['ServiceConnect'])
            self.jobs.load()
            self.jobs.main_chart()
            self.jobs.new_history_chart()
            sys.exit(-1)
            #print(self.jobs.closed_history)
        else:
            self.jobs = None
        if self.config['Projects']['active']:
            self.projects = Projects(self, self.config['Projects'])
            self.projects.load()
            self.projects.chart()
        else:
            self.projects = None

    
    def render(self):
        renderer = pystache.Renderer(search_dirs=self.cf['templates'])
        outfile = os.path.join(self.cf['outdir'], self.cf['report'])
        with open(outfile, 'w') as f:
            f.write(renderer.render(self))
            print("Wrote report to %s" % outfile)
        #print(renderer.render(self.projects))




if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--config', required=True, type=str, help="Config file")
    args = parser.parse_args()
    report = HighlightsReport(args.config)
    report.load()
    report.render()

