import re, datetime

# Email & directory stuff
import imaplib
import pyzmail
import ldap3

# Build a graph - not using this ATM, but maybe one day
import networkx as nx

# The usual data and plotting libraries
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
import json
import os, os.path
import subprocess

class Interactions(object):
    '''
    TODO doc 
    '''
    def __init__(self, args):
        server =  ldap3.Server(args.ldap, get_info=ldap3.ALL)
        self.ldap_connection = ldap3.Connection(server, os.environ['LDAP_USER'] , \
            os.environ['LDAP_PASSWD'], auto_bind=True)
        self.args = args
        self.dept_lookup = json.load(open(args.mapping))  if args.mapping else {}
        self.M = imaplib.IMAP4_SSL(args.server)
        self.M.login(args.username, args.password)
        self.M.select('"%s"' % self.args.folder)
        self.interactions = nx.MultiGraph()
        self.sent_to_summary = {}

    def lookup_dept(self, dept):
        return self.dept_lookup[dept] if  dept in self.dept_lookup else dept

    def lookup_ldap_unit(self, from_email):
        self.ldap_connection.search('o=UTS', '(mail=%s)' % from_email, \
            attributes=[self.args.unit_string, self.args.unit_string2])
        if len(self.ldap_connection.entries):
            unit =  self.ldap_connection.entries[0][self.args.unit_string] or \
                self.ldap_connection.entries[0][self.args.unit_string2]
        else:
            unit = "EXTERNAL"
        if not unit and from_email.endswith("@student.uts.edu.au"):
                unit = "Students (unknown)"
        if type(unit) is str:
            unit = [unit]
        return [self.lookup_dept(u) for u in unit]

    def deal_with_the_emails(self):
        typ, data = self.M.search(None, '(%s)' % self.args.query)
        for num in data[0].split():
            #(UID BODY[TEXT])
            # http://stackoverflow.com/questions/19540192/imap-get-sender-name-and-body-text
            typ, data = self.M.fetch(num, '(RFC822)')
            # fetch the email body (RFC822) 
            raw_email = data[0][1]
            message = pyzmail.PyzMessage.factory(raw_email)

            #print("::SUBJECT::", message.get_subject(), end="")
            if not self.args.ignore_headers:
                from_name, from_email = message.get_address('from')
                # If we're looking at sent mail, we want to be able to force 
                # the name of the sending unit. 
                if self.args.from_unit:
                    from_unit = self.args.from_unit
                else:
                    from_unit = self.lookup_ldap_unit(from_email)[0]

                from_email = from_email.lower()
                self.interactions.add_node(from_unit)
                for (to_name,to_email) in message.get_addresses('to') + message.get_addresses('cc'):
                    to_email = to_email.lower()
                    to_units = self.lookup_ldap_unit(to_email)
                    print ("Considering email to %s @ %s" % (to_email, to_units))
                    for to_unit in [t for t in to_units if t]:
                        if to_unit in self.sent_to_summary:
                            self.sent_to_summary[to_unit] += 1
                        else:
                            self.sent_to_summary[to_unit] = 1
                        self.interactions.add_node(to_unit)
                        self.interactions.add_edge(from_unit, to_unit)

                print("")

    def report(self):
        """ Output data files """
        
        #nx.draw_spring(self.interactions,with_labels=True)
        #plt.show()

        interactions_frame = nx.to_pandas_dataframe(self.interactions)
        print(interactions_frame)
        to_series = pd.Series(self.sent_to_summary)
        df = pd.DataFrame()
        for dept, count in to_series.items():
            df.set_value(self.args.username, dept, count)
        
        print(df)
        save_to_dir = os.path.join(".", "Output", self.args.query, \
            self.args.username.replace("@","_at_"))
        if not os.path.exists(save_to_dir):
            os.makedirs(save_to_dir)
     
        df.to_csv(os.path.join(save_to_dir, "interaction_data.csv"))
        to_series.plot(kind='bar')
        plt.tight_layout()
        img_path = os.path.join(save_to_dir, "interaction_data.png")
        plt.savefig(img_path)

        # The application "open" only exists on Mac OSX. It needs to be 
        # commented out if running on Linux.
        #subprocess.call(["open", img_path])

        # This part saves the interaction image to a directory which can be read by the web app.
        # The filename will be like: static/output/date_range/users_email   
        # e.g. static/output/2018.05.01-2018.31.03/Mike.Lake_at_uts.edu.au/
        (start_date, end_date) = self.imap_to_iso_dates()
        save_to_dir = 'static/output/%s-%s/%s' % (start_date.strftime('%Y.%m.%d'), \
            end_date.strftime('%Y.%m.%d'), self.args.username.replace("@","_at_"))
        if not os.path.exists(save_to_dir):
            os.makedirs(save_to_dir)
        img_path = os.path.join(save_to_dir, "interaction_data.png")
        plt.savefig(img_path)

        return img_path

    def imap_to_iso_dates(self):
        '''
        Changes the IMAP date format to an ISO date format e.g. 
        "SINCE 01-Apr-2017 BEFORE 03-May-2017" to "2017.05.01" and "2017.05.07"
        '''
        (since, start_str, before, end_str) = self.args.query.split(' ')
        start_date = datetime.datetime.strptime(start_str, '%d-%b-%Y')
        end_date = datetime.datetime.strptime(end_str, '%d-%b-%Y')
        # TODO: Check if start_date < end_date and take action if not.

        return (start_date, end_date)
        
    def close(self):
        self.M.close()
        self.M.logout()

